import sys
import traceback
import argparse
import requests
from bs4 import BeautifulSoup

class RemoteTests:
    def __init__(self, user, host):
        self.user = user
        self.host = host

    def req(self, method, url=None, uri=None, *args, **kwargs):
        if url is None and uri is not None:
            if 'protocol' in args:
                protocol = kwargs['protocol']
                del kwargs['protocol']
            else:
                protocol = 'https'

            url = f'{protocol}://{self.host}/{uri}'

        return requests.request(method, url, *args, **kwargs)

    def try_0010_host_meta(self):
        r = self.req(method = 'GET',
                     uri = '/.well-known/host-meta',
                     #accept = 'application/xrd+xml',
                     )

        compare(r.status_code, '==', 200)
        compare(r.headers['Content-Type'], '==',
                'application/xrd+xml')

        content = BeautifulSoup(r.text, features='xml')

        compare (content, '==', 'banana')

    def try_0020_not_gatewayed(self):
        r = self.req(method = 'GET',
                     uri = '/.kepi-not-gatewayed',
                     )
        compare(r.status_code, '==', 200)

    def run(self):
        print('Looking up')
        for name in dir(self):
            if not name.startswith('try_'):
                continue

            print('%40s - ' % (name,),
                  end='',
                  flush = True,
                  )

            try:
                result = getattr(self, name)()
                success = True
            except Exception:
                _, _, tb = sys.exc_info()
                stack = traceback.extract_tb(tb)
                result = repr(stack[-1].line)
                success = False

            if success:
                print('ok')
            else:
                print('FAIL')

            if result!=None:
                print(result)

def main():
    parser = argparse.ArgumentParser(
            description='test remote kepi configuration')
    parser.add_argument(
            'username',
            help='name of user to look up')
    args = parser.parse_args()

    username = args.username
    if username.startswith('@'):
        username = username[1:]

    fields = username.split('@')
    if len(fields)!=2:
        raise ValueError("usernames should be in the form @user@host")

    tests = RemoteTests(
            user = fields[0],
            host = fields[1],
            )
    tests.run()

if __name__=='__main__':
    main()
