import sys
import os
import argparse
import logging
import json

import kepi.validate
import kepi.fastcgi
from kepi.config import config

logger = logging.getLogger('kepi')
logging.basicConfig(
        level = logging.INFO,
        stream = sys.stdout,
        )

COMMANDS = [
        'validate',
        'fastcgi',
        ]

def daemonise(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):

    pid = os.fork()
    if pid > 0:
        sys.exit(0)

    os.chdir('/')
    os.umask(0)
    os.setsid()

    pid = os.fork()
    if pid > 0:
        sys.exit(0)

    for f, mode, buffering in [
            (sys.stdin,  'r',  -1),
            (sys.stdout, 'a+', -1),
            (sys.stderr, 'a+',  0),
            ]:

        if mode!='r':
            f.flush()

        i = open(f, mode, buffering)
        os.dup2(i.fileno( ), f.fileno( ))

    logger.info("Running at PID %s", os.getpid())

def load_message(name):
    if name=='-':
        f = sys.stdin
    else:
        f = open(name, 'r')

    result = json.load(f)

    if f!=sys.stdin:
        f.close()

    return result

def main():
    parser = argparse.ArgumentParser(
            prog='kepi',
            description='send or receive ActivityPub messages',
            )

    parser.add_argument(
            '--spool', '-S', default='/var/spool/kepi',
            help='directory to store the messages')

    config.parse_args(parser)

    for command in COMMANDS:
        getattr(kepi, command).add_argparser(subparsers)

    args = parser.parse_args()
    config.handle_argparse(args)

    getattr(kepi, args.command).run()

if __name__=='__main__':
    main()
