import argparse
import os
import kepi.users

DEFAULT_CONFIG_FILENAME = '/etc/kepi/kepi.conf'
CONFIG_SECTION = 'kepi'

class ConfigUsers:
    def __init__(self):
        raise NotImplementedError()

    def __getitem__(self, who):

        assert '/' not in who
        assert not who.startswith('.')

        filename = os.path.join(
                config.users_dir,
                who,
                f'{who}.json',
                )
        return kepi.users.User(filename)

class Config:

    settings = {}
    subcommands = []
    argparser = None

    users = ConfigUsers.__new__(ConfigUsers)

    def __init__(self):
        raise NotImplementedError()

    def __getattr__(self, field):
        if field in self.settings:
            return self.settings[field]
        else:
            raise AttributeError(field)

    def __setattr__(self, field, value):
        self.settings[field] = value

    def parse_args(self, argparser):

        assert self.subcommands is not None

        self.argparser = argparser

        self.subparsers = argparser.add_subparsers(
                dest = 'command',
                required = True,
                )

        for sub in self.subcommands:
            sub(self.subparsers)

        args = self.argparser.parse_args()

        for field in dir(args):
            if field.startswith('_'):
                continue
            self.settings[field] = getattr(args, field)

config = Config.__new__(Config)

def subcommand(fn):
    """
    Decorator
    """
    config.subcommands.append(fn)
    return None

def normalise_url(
        address,
        env = {},
        ):

    # For now, a naive implementation
    return f'https://{env["SERVER_NAME"]}{address}'
