def flavour_to_message(flavour):

    assert 'type' in flavour, flavour
    result = {}

    if flavour['type']=='follow':
        result = {
                "actor": "https://them.example.org/users/them",
                "id": "https://them.example.org/c9876",
                "object": "https://us.example.org/users/us",
                "type": "Follow",
                }
    elif flavour['type']=='unfollow':
        result = {
                "actor": "https://us.example.org/users/them",
                "id": "https://us.example.org/c5678",
                "object": {
                    "actor": "https://them.example.org/users/them",
                    "id": "https://them.example.org/c9876",
                    "object": "https://us.example.org/users/us",
                    "type": "Follow",
                    },
                "type": "Undo",
                }
    elif flavour['type']=='post':
        result = {
                "actor": flavour['from'],
                "to": [
                    "https://www.w3.org/ns/activitystreams#Public",
                    ],
                "cc": [
                    ],
                "id": flavour['activity-id'],
                "object": {
                    "atomUri": flavour['atom'],
                    "attachment": [],
                    "attributedTo": flavour['from'],
                    "cc": [
                        ],
                    "content": (
                        flavour['content'],
                        ),
                    "contentMap": {
                        # XXX do this properly
                        "en": flavour['content'],
                        },
                    "conversation": (
                        "tag:them.example.org,2019-04-04:objectId=13169876:"
                        "objectType=Conversation"
                        ),
                    "id": flavour['content-id'],
                    "published": flavour['date'],
                    "sensitive": False,
                    "summary": None,
                    "tag": [
                        ],
                    "to": [
                        "https://www.w3.org/ns/activitystreams#Public"
                        ],
                    "type": "Note",
                    "url": flavour['content-id'],
                    },
                "published": flavour['date'],
                "type": "Create",
                }
    else:
        raise ValueError(
                f"Unknown flavour type: {flavour['type']}"
                )

    return result

def message_to_flavour(message):

    assert 'type' in message, message

    if message['type']=='Create':
        inner = message['object']

        # XXX Inner assertions should just be warnings
        sender = message['actor']
        if 'attributedTo' in inner:
            assert inner['attributedTo']==sender

        assert inner['id'] == inner['url']

        return {
                'flavour': 'post',
                'activity-id': message['id'],
                'atom': inner.get('atomUri'),
                'from': sender,
                'content': inner['content'],
                'content-id': inner['id'],
                }
    elif message['type']=='Follow':
        raise ValueError()
    elif message['type']=='Undo':
        raise ValueError()
    else:
        raise ValueError(
                f"Unknown message type: {message['type']}"
                )
